@@ -11,10 +11,12 @@ from typing import Any, Dict, List, Optional, Type
 from uuid import UUID
 
 import aerospike
+import aiohttp
 from faust import Topic
 from kafka import KafkaAdminClient, KafkaConsumer, KafkaProducer
 from kafka.admin import NewTopic
 
+from dagger.modeler.builder_helper import DAGBuilderHelper
 from dagger.modeler.definition import (
     DefaultTemplateBuilder,
     DynamicProcessTemplateDagBuilder,
@@ -55,6 +57,7 @@ from dagger.templates.template import (
     ParallelCompositeTaskTemplate,
     ParallelCompositeTaskTemplateBuilder,
     TriggerTaskTemplateBuilder,
+    IProcessTemplateDAGBuilder,
 )
 
 KAFKA_ADMIN_CLIENT_URL = "kafka:29092"
@@ -103,6 +106,9 @@ try:
     topic_list.append(
         NewTopic(name="simple_topic", num_partitions=1, replication_factor=1)
     )
+    topic_list.append(
+        NewTopic(name="simple_topic_stop", num_partitions=1, replication_factor=1)
+    )
     topic_list.append(
         NewTopic(name="SIMPLE_LISTENER", num_partitions=1, replication_factor=1)
     )
@@ -136,6 +142,8 @@ workflow_engine = Dagger(
 workflow_engine.tables_cleared = False
 orders_topic = workflow_engine.faust_app.topic(ORDERS_TOPIC, value_type=str)
 simple_topic = workflow_engine.faust_app.topic("simple_topic", value_type=str)
+simple_topic_stop = workflow_engine.faust_app.topic("simple_topic_stop", value_type=str)
+
 simple_listener = workflow_engine.faust_app.topic("SIMPLE_LISTENER", value_type=str)
 
 templates: List[ITemplateDAGInstance] = list()
@@ -195,6 +203,22 @@ class PaymentKafkaCommandTask(KafkaCommandTask[str, str]):
         logger.info(f"Sending Payment Command to topic {self.topic}")
 
 
+class OrderCommandTask(KafkaCommandTask[str, str]):
+    async def execute(
+        self,
+        runtime_parameters: Dict[str, str],
+        workflow_instance: ITemplateDAGInstance,
+    ) -> None:
+        payload = {
+            "order_id": runtime_parameters["order_id"],
+            "customer_id": runtime_parameters["customer_id"],
+            "pizza_type": runtime_parameters["pizza_type"],
+        }
+        topic: Topic = await workflow_engine.topics[self.topic].send(
+            value=json.dumps(payload)
+        )
+
+
 class PaymentDecisionTask(DecisionTask[str, str]):
     async def evaluate(self, **kwargs: Any) -> Optional[UUID]:
         for task_id in self.next_dags:
@@ -208,6 +232,19 @@ class FulfillmentKafkaCommandTask(KafkaCommandTask[str, str]):
         logger.info(f"Sending Command to topic {self.topic}")
 
 
+class DeliveryCommandTask(ExecutorTask[str, str]):
+    async def execute(
+        self, runtime_parameters: Dict[str, VT], workflow_instance: ITask = None
+    ) -> None:
+        payload = {
+            "order_id": runtime_parameters["order_id"],
+            "customer_id": runtime_parameters["customer_id"],
+        }
+        async with aiohttp.ClientSession() as session:
+            async with session.post(url="http://www.deliverysvc.com", json=payload):
+                pass
+
+
 class FulfillmentTriggerTask(TriggerTask[str, str]):
     async def execute(
         self, runtime_parameters: Dict[str, VT], workflow_instance: ITask
@@ -262,12 +299,67 @@ class PaymentKafkaListenerTask(KafkaListenerTask[str, str]):
         return self.correlatable_key, tpayload[self.correlatable_key]
 
 
+class PizzaWaitForReadyListener(KafkaListenerTask[str, str]):
+    correlatable_key = "order_id"
+
+    async def get_correlatable_keys_from_payload(
+        self, payload: Any
+    ) -> List[TaskLookupKey]:
+        tpayload = json.loads(payload)
+        key = tpayload[self.correlatable_key]
+        return [(self.correlatable_key, key)]
+
+    async def on_message(
+        self, runtime_parameters: Dict[str, VT], *args: Any, **kwargs: Any
+    ) -> bool:
+        logger.info(f"Pizza Order is Ready")
+        return True
+
+
+def pizza_ordering_process(process_name: str = "Order") -> IProcessTemplateDAGBuilder:
+    dag_builder = DAGBuilderHelper(dagger_app=workflow_engine)
+    root_task = dag_builder.build_and_link_tasks(
+        [
+            dag_builder.generic_command_task_builder(
+                topic="pizza_order_topic",
+                task_type=OrderCommandTask,
+                process_name=process_name,
+            ),
+            dag_builder.generic_listener_task_builder(
+                topic="PizzaWaitForReadyListener",
+                task_type=PizzaWaitForReadyListener,
+                process_name=process_name,
+            ),
+        ]
+    )
+    return dag_builder.generic_process_builder(
+        process_name=process_name, root_task=root_task
+    )
+
+
+def pizza_delivery_process(
+    process_name: str = "Delivery",
+) -> IProcessTemplateDAGBuilder:
+    dag_builder = DAGBuilderHelper(dagger_app=workflow_engine)
+    root_task = dag_builder.build_and_link_tasks(
+        [
+            dag_builder.generic_executor_task_builder(
+                task_type=DeliveryCommandTask,
+                name=process_name,
+            )
+        ]
+    )
+    return dag_builder.generic_process_builder(
+        process_name=process_name, root_task=root_task
+    )
+
+
 class SimpleKafkaListenerTask(KafkaListenerTask[str, str]):
 
     correlatable_key = "simple_id"
 
     async def stop(self) -> None:
-        pass
+        print("Stop called")
 
     def get_status(self) -> TaskStatus:
         return self.status
@@ -390,6 +482,20 @@ def register_simple_process(process_name: str) -> IProcessTemplateDAG:
     return simple_process_builder.build()
 
 
+@Dagger.register_template("PizzaWorkflow")
+def register_pizza_workflow(template_name: str) -> ITemplateDAG:
+    dag_builder_helper = DAGBuilderHelper(workflow_engine)
+    order_process = dag_builder_helper.build_and_link_processes(
+        [
+            pizza_ordering_process(process_name="Order"),
+            pizza_delivery_process(process_name="Delivery"),
+        ]
+    )
+    return dag_builder_helper.generic_template(
+        template_name=template_name, root_process=order_process
+    )
+
+
 @Dagger.register_template("OrderWorkflow")
 def test_order_template(template_name: str) -> ITemplateDAG:
     payment_status_topic: Topic = Dagger.create_topic(
@@ -590,6 +696,28 @@ def test_order_template(template_name: str) -> ITemplateDAG:
     return template_builder.build()
 
 
+async def create_and_submit_pizza_delivery_workflow(
+    order_id: str, customer_id: str, pizza_type: int
+):
+    pizza_workflow_template = workflow_engine.template_dags["PizzaWorkflow"]
+    pizza_workflow_instance = await pizza_workflow_template.create_instance(
+        uuid.uuid1(),
+        repartition=False,  # Create this instance on the current worker
+        order_id=order_id,
+        customer_id=customer_id,
+        pizza_type=pizza_type,
+    )
+    await workflow_engine.submit(pizza_workflow_instance, repartition=False)
+
+
+@workflow_engine.faust_app.agent(simple_topic_stop)
+async def simple_data_stream_stop(stream):
+    async for value in stream:
+
+        instance = await workflow_engine.get_instance(running_task_ids[-1])
+        await instance.stop()
+
+
 @workflow_engine.faust_app.agent(simple_topic)
 async def simple_data_stream(stream):
     async for value in stream:
